/***************************************************/
/**** MCMC application for Missing data         ****/
/***************************************************/
/**** Programmed by : Jaeil                     ****/
/**** Begin Date    : 08.20.08                  ****/
/**** Last Update   : 09.25.08                  ****/
/***************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include <math.h>
#include <float.h>
#include <ctype.h>
#include <assert.h>


#define Niter	1
#define MAX_ITER   100000
#define _MAX_BUF_    2000
#define BURN    20000

/* ARMS use */
//#define RAND_MAX 2147483647      /* For Sun4 : remove this for some systems */ 
#define XEPS  0.00001            /* critical relative x-value difference */
#define YEPS  0.1                /* critical y-value difference */
#define EYEPS 0.001              /* critical relative exp(y) difference */
#define YCEIL 50.                /* maximum y avoiding overflow in exp(y) */

/* ******ARMS */
typedef struct point {    /* a point in the x,y plane */
  double x,y;             /* x and y coordinates */
  double ey;              /* exp(y-ymax+YCEIL) */
  double cum;             /* integral up to x of rejection envelope */
  int f;                  /* is y an evaluated point of log-density */
  struct point *pl,*pr;   /* envelope points to left and right of x */
} POINT;

/* *********************************************************************** */

typedef struct envelope {  /* attributes of the entire rejection envelope */
  int cpoint;              /* number of POINTs in current envelope */
  int npoint;              /* max number of POINTs allowed in envelope */
  int *neval;              /* number of function evaluations performed */
  double ymax;             /* the maximum y-value in the current envelope */
  POINT *p;                /* start of storage of envelope POINTs */
  double *convex;          /* adjustment for convexity */
} ENVELOPE;

/* *********************************************************************** */

typedef struct funbag { /* everything for evaluating log density          */
  double (*myfunc)(double x, int opt);
                     /* user-defined function evaluating log density at x */
} FUNBAG;

/* *********************************************************************** */

typedef struct metropolis { /* for metropolis step */
  int on;            /* whether metropolis is to be used */
  double xprev;      /* previous Markov chain iterate */
  double yprev;      /* current log density at xprev */
} METROPOLIS;

/* *********************************************************************** */

typedef struct tagPARAM{
	int n;
  int np, nq;			// size of X, size of Z
  int nb;
  int ksize;
  
  int nbeta;			// size of beta (x+z);
  
  /*Normal distribution */
  double sigma;
  double *beta1;
  double *beta2;
  double *gamma;
  double *phi;
  
  double *tmpZ;

} PARAM;

typedef struct tracePARAM{
	int n;
  int np, nq;			// size of X, size of Z
  int nb;
  int ksize;
  
  int nbeta;			// size of beta (x+z);
  
  /*Normal distribution */
  double **beta1;
  double **beta2;
  double **gamma;
  double **phi;
	double *DIC;

} TRPARAM;

typedef struct tagData{
	int ntot;
	int nN;
	int nM;
	int p;
	
  double **X;
  int *Y;
} DATA;

typedef struct tagPRIOR{
  int n;       
  int np, nq;
  int nb;
  int ksize;
  
  double *Mbeta1;
  double Vbeta1;

  double *Mbeta2;
  double Vbeta2;
  double *Gprob;
	double *MPhi;
	double VPhi;
  /* Does not contain gamma prior because Dirichlet (1,1,1,...1) is just Uniform*/
  
} PRIOR;

/*************/
/* functions */
/*************/
void res_print(int niter);
void res_print2();

/****************   init_data.c  ***************************/
void read_data(FILE *fp,  int *nN, int *nM, int *p);
void init_param(int *p, int *k, int *nstr, int *M);
void init_trparam(int *p, int *k, int *nstr, int *M);
void init_prior(int *p, int *k, int *nstr, int *M);
void init_DATA(int *nstr, int *M, int *p);
void init_values(int *p, int *k, int *nstr, int *M);
void free_param(int *nstr) ;
void free_data(int *nstr, int *M);
void free_trparam(int *nstr, int *M);
void free_prior();
void read_mle();

void data_allocation2(int *ncase, int *ratio, int nstep);

void data_save(int niter, int nncase, int ratio);

/****************   update.c   *****************************/
void value_trans(int nstep);
void setPhi();
void mhgibbs(int nsimul);
double Calc_DIC(int nstep);

double arms_beta1(double x, int nopt);
double arms_beta2(double x, int nopt);
double arms_kappa(double x, int nopt);
double calcGammaMH(int nstep);

/****************   mmath.c   ******************************/
double ran1(long idum);
double drand48 (void);
double rexp (double scale);
double rnormal(double mu, double sigma);
double rnorm (double mu, double sigma);
//void rdirichlet(double *val, double *prob, int len);
void rdirichlet();
double vec2_sum(const double *vec1, const double *vec2, int len);
double vec_sum(const double *vec, int len);
int vec_max(const int *vec, int len);
double vec_min(const double *vec, int len);
double vec_mean(const double *vec, const int len);
double vec_mean2(const double *vec, int start, int len);
double vec_var(const double *vec, int len);


/****************   ARMS   ******************************/
int pseudomain(void);

int arms (double *xinit, int ninit, double *xl, double *xr,
	 double (*myfunc)(double x, int opt), double *convex, int npoint, int dometrop, double *xprev, double *xsamp,
         int nsamp, double *qcent, double *xcent, int ncent, int *neval, int _opt);

double expshift(double y, double y0);

int initial (double *xinit, int ninit, double xl, double xr, int npoint,
	     FUNBAG *lpdf, ENVELOPE *env, double *convex, int *neval,
             METROPOLIS *metrop);
void sample(ENVELOPE *env, POINT *p);
void invert(double prob, ENVELOPE *env, POINT *p);
int test(ENVELOPE *env, POINT *p, FUNBAG *lpdf, METROPOLIS *metrop);
int update(ENVELOPE *env, POINT *p, FUNBAG *lpdf, METROPOLIS *metrop);
void cumulate(ENVELOPE *env);
int meet (POINT *q, ENVELOPE *env, METROPOLIS *metrop);
double area(POINT *q);
double expshift(double y, double y0);
double logshift(double y, double y0);
double perfunc(FUNBAG *lpdf, ENVELOPE *env, double x);
void display(FILE *f, ENVELOPE *env);
double u_random();



//* Structure Variable define*/
DATA data; 
PARAM param;
PRIOR prior;
TRPARAM trparam;

